local ANIM_SMOKE_TEXTURE = resolvefilepath("fx/animsmoke.tex")

local REVEAL_SHADER = resolvefilepath("shaders/vfx_particle_reveal.ksh")

local COLOUR_ENVELOPE_NAME = "cane_sharp_colourenvelope"
local SCALE_ENVELOPE_NAME = "cane_sharp_scaleenvelope"

local assets =
{
    Asset("IMAGE", ANIM_SMOKE_TEXTURE),
    Asset("SHADER", REVEAL_SHADER),
}

--------------------------------------------------------------------------

local function IntColour(r, g, b, a)
    return { r / 255, g / 255, b / 255, a / 255 }
end

local function InitEnvelope()
    EnvelopeManager:AddColourEnvelope(
        COLOUR_ENVELOPE_NAME,
        {
            { 0,    IntColour(75, 0, 130, 160) },
            { .19,  IntColour(75, 0, 130, 160) },
            { .35,  IntColour(75, 0, 130, 80) },
            { .51,  IntColour(75, 0, 130, 60) },
            { .75,  IntColour(75, 0, 130, 40) },
            { 1,    IntColour(75, 0, 130, 0) },
        }
    )

    local glow_max_scale = .21
    EnvelopeManager:AddVector2Envelope(
        SCALE_ENVELOPE_NAME,
        {
            { 0,    { glow_max_scale * 0.7, glow_max_scale * 0.7 } },
            { .55,  { glow_max_scale * 1.2, glow_max_scale * 1.2 } },
            { 1,    { glow_max_scale * 1.3, glow_max_scale * 1.3 } },
        }
    )

    InitEnvelope = nil
    IntColour = nil
end

--------------------------------------------------------------------------
local GLOW_MAX_LIFETIME = 2.1

local function emit_glow_fn(effect, emitter_fn)
    local vx, vy, vz = .005 * UnitRand(), 0, .005 * UnitRand()
    local lifetime = GLOW_MAX_LIFETIME * (.9 + math.random() * .1)
    local px, py, pz = emitter_fn()

    effect:AddRotatingParticle(
        lifetime,           -- lifetime
        px, py, pz,         -- position
        vx, vy, vz,         -- velocity
        math.random() * 360,-- angle
        UnitRand() * 1     -- angle velocity
    )
end

	
local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()

    inst:AddTag("FX")

    inst.persists = false

	inst:AddTag("INTERIOR_LIMBO_IMMUNE")
    if InitEnvelope ~= nil then
        InitEnvelope()
    end

    local effect = inst.entity:AddParticleEmitter()

    effect:SetRenderResources(ANIM_SMOKE_TEXTURE, REVEAL_SHADER)
    effect:SetMaxNumParticles(128)
    effect:SetRotationStatus(true)
    effect:SetMaxLifetime(GLOW_MAX_LIFETIME)
    effect:SetColourEnvelope(COLOUR_ENVELOPE_NAME)
    effect:SetScaleEnvelope(SCALE_ENVELOPE_NAME)
    effect:SetBlendMode(BLENDMODE.AlphaBlended)
    effect:EnableBloomPass(true)
    effect:SetSortOrder(0)
    --effect:SetSortOffset(0, 0)
    --effect:SetKillOnEntityDeath(0, true)
    --effect:SetFollowEmitter(0, true)

    -----------------------------------------------------


    local tick_time = TheSim:GetTickTime()

    local glow_desired_pps = 3
    local glow_particles_per_tick = glow_desired_pps * tick_time
    local glow_num_particles_to_emit = 0

    local sphere_emitter = CreateSphereEmitter(.05)
    
    EmitterManager:AddEmitter(inst, nil, function()

        while glow_num_particles_to_emit > 1 do
            emit_glow_fn(effect, sphere_emitter)
            glow_num_particles_to_emit = glow_num_particles_to_emit - 1
        end
        glow_num_particles_to_emit = glow_num_particles_to_emit + glow_particles_per_tick * math.random() * 3

    end)

    return inst
end

return Prefab("cane_sharp_fx", fn, assets)
