local assets =
{
	Asset("ANIM", "anim/lantern.zip"),
	Asset("ANIM", "anim/swap_lantern.zip"),
    Asset("SOUND", "sound/wilson.fsb"),
    Asset("INV_IMAGE", "lantern_lit"),
}
local lantern_skins =
	{	
		{
			skin = 	"lantern",  
			xml = 	"images/inventoryimages.xml", 
			tex =	"lantern.tex",
			desc = 	"Classic",   
		},
		{
			skin = 	"lantern_tesla",  
			build = "swap_lantern_off",
			xml = 	"images/inventoryimages/lantern_tesla.xml", 
			tex =	"lantern_tesla.tex",
			desc = 	"Tesla",  
		},
        {
			skin = 	"lantern_winter",  
			build = "swap_lantern_off",
			xml = 	"images/inventoryimages/lantern_winter.xml", 
			tex =	"lantern_winter.tex",
			desc = 	"Winter",  
		},			
	}
local function lantern_enterlimbo(inst)
    if inst._lit_fx ~= nil then
        inst._lit_fx._lastpos = inst._lit_fx:GetPosition()
        local parent = inst.entity:GetParent()
        if parent ~= nil then
            local x, y, z = parent.Transform:GetWorldPosition()
            local angle = (360 - parent.Transform:GetRotation()) * DEGREES
            local dx = inst._lit_fx._lastpos.x - x
            local dz = inst._lit_fx._lastpos.z - z
            local sinangle, cosangle = math.sin(angle), math.cos(angle)
            inst._lit_fx._lastpos.x = dx * cosangle + dz * sinangle
            inst._lit_fx._lastpos.y = inst._lit_fx._lastpos.y - y
            inst._lit_fx._lastpos.z = dz * cosangle - dx * sinangle
        end
    end
end

local function lantern_off(inst)
    local fx = inst._lit_fx
    if fx ~= nil then
        if fx.KillFX ~= nil then
            inst._lit_fx = nil
            fx:RemoveEventCallback("enterlimbo", lantern_enterlimbo, inst)
            fx._lastpos = fx._lastpos or fx:GetPosition()
            fx.entity:SetParent(nil)
            if fx.Follower ~= nil then
                fx.Follower:FollowSymbol(0, "", 0, 0, 0)
            end
            fx.Transform:SetPosition(fx._lastpos:Get())
            fx:KillFX()
        else
            fx:Remove()
			inst._lit_fx = nil
        end
    end
end

local function lantern_on(inst,skin)
    if inst._lit_fx ~= nil then
        inst._lit_fx:Remove()
    end
    local owner = inst.components.inventoryitem.owner
    if owner ~= nil then
			if skin ==  "lantern_tesla" then
				inst._lit_fx = SpawnPrefab("lantern_tesla_fx_held")
				local follower = inst._lit_fx.entity:AddFollower()
				inst._lit_fx:AddTag("INTERIOR_LIMBO_IMMUNE")
				follower:FollowSymbol( owner.GUID, "swap_object", 67, -7, 0)
			else
                inst._lit_fx = SpawnPrefab("lantern_winter_fx_held")
				inst._lit_fx:AddTag("INTERIOR_LIMBO_IMMUNE")
				inst._overridesymbols = {"snowflake"}
                if inst._overridesymbols ~= nil and inst._lit_fx.AnimState ~= nil then
                    for i, v in ipairs(inst._overridesymbols) do
                        inst._lit_fx.AnimState:OverrideSymbol(v, "lantern_winter_fx", v)
                    end
                end
                inst._lit_fx.entity:AddFollower()
				inst._lit_fx.Follower:FollowSymbol(owner.GUID, "swap_object", 67, -7, 0)	
			end
    else
			if skin ==  "lantern_tesla" then
				inst._lit_fx = SpawnPrefab("lantern_tesla_fx_ground")
				local follower = inst._lit_fx.entity:AddFollower()
				inst._lit_fx:AddTag("INTERIOR_LIMBO_IMMUNE")
				follower:FollowSymbol( inst.GUID, "lantern01", 0, 60, 0)
			else
            inst._lit_fx = SpawnPrefab("lantern_winter_fx_ground")
				inst._overridesymbols = {"snowflake"}
                if inst._overridesymbols ~= nil and inst._lit_fx.AnimState ~= nil then
                    for i, v in ipairs(inst._overridesymbols) do
                        inst._lit_fx.AnimState:OverrideSymbol(v, "lantern_winter_fx", v)
                    end
                end
                if inst._lit_fx.KillFX ~= nil then
                    inst._lit_fx:ListenForEvent("enterlimbo", lantern_enterlimbo, inst)
					local follower = inst._lit_fx.entity:AddFollower()
					inst._lit_fx:AddTag("INTERIOR_LIMBO_IMMUNE")
					follower:FollowSymbol( inst.GUID, "lantern01", 0, 60, 0)
                end
            end     
    end
end

local function lightup(inst)
    if inst.components.fueled then
        inst.components.fueled:StartConsuming()        
    end
    inst.Light:Enable(true)
	if SaveGameIndex:IsModeShipwrecked() or  SaveGameIndex:GetCurrentMode()== "porkland" then
		inst.components.floatable:UpdateAnimations("idle_on_water", "idle_on")
		else
		inst.AnimState:PlayAnimation("idle_on")
	end
	local build = inst.components.dsskins.skin ~= nil and inst.components.dsskins.skin or "lantern"
    if inst.components.equippable:IsEquipped() then
        inst.components.inventoryitem.owner.AnimState:OverrideSymbol("swap_object", "swap_"..build, "swap_lantern_on")
        inst.components.inventoryitem.owner.AnimState:Show("LANTERN_OVERLAY") 
    end
    inst.components.machine.ison = true

    inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_on")
    inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_LP", "loop")
	if build ~= "lantern" then
		lantern_on(inst,build)
		inst.components.inventoryitem.atlasname = "images/inventoryimages/"..build.."_lit.xml"
	end
    inst.components.inventoryitem:ChangeImageName(build.."_lit")
end

local function turnon(inst)
    if not inst.components.fueled:IsEmpty() then
        if not inst.components.machine.ison then
            lightup(inst)
        end
    end
end

local function turnoff(inst)
    if inst.components.fueled then
        inst.components.fueled:StopConsuming()        
    end

    inst.Light:Enable(false)
	if SaveGameIndex:IsModeShipwrecked() or  SaveGameIndex:GetCurrentMode()== "porkland" then
		inst.components.floatable:UpdateAnimations("idle_off_water", "idle_off")
		else
		inst.AnimState:PlayAnimation("idle_off")
	end
	local build = inst.components.dsskins.skin ~= nil and inst.components.dsskins.skin or "lantern"
    if inst.components.equippable:IsEquipped() then
		
        inst.components.inventoryitem.owner.AnimState:OverrideSymbol("swap_object", "swap_"..build, "swap_lantern_off")
        inst.components.inventoryitem.owner.AnimState:Hide("LANTERN_OVERLAY") 
    end

    inst.components.machine.ison = false

    inst.SoundEmitter:KillSound("loop")
    inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_off")
	if build ~= "lantern" then
		inst.components.inventoryitem.atlasname = "images/inventoryimages/"..build..".xml"
	end
    inst.components.inventoryitem:ChangeImageName(build)
	lantern_off(inst)
end

local function refreshstatus(inst)
    if inst.components.fueled:IsEmpty() then
        turnoff(inst)
    else
        turnon(inst)
    end
end

local function OnLoad(inst, data)
    if inst.components.machine and inst.components.machine.ison then
        lightup(inst)
    else
        turnoff(inst)
    end
end

local function ondropped(inst)
	inst:DoTaskInTime(0.1, function()
	turnoff(inst)
	turnon(inst)
	end)
end

local function onpickup(inst)
	lightup(inst)
end

local function onputininventory(inst)
    turnoff(inst)
end

local function onequip(inst, owner)
    refreshstatus(inst)
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal")
	local build = inst.components.dsskins.skin ~= nil and inst.components.dsskins.skin or "lantern"
    owner.AnimState:OverrideSymbol("lantern_overlay", "swap_"..build, "lantern_overlay")
	
    if inst.components.fueled:IsEmpty() then
        owner.AnimState:OverrideSymbol("swap_object", "swap_"..build, "swap_lantern_off")
		owner.AnimState:Hide("LANTERN_OVERLAY") 
    else
        owner.AnimState:OverrideSymbol("swap_object", "swap_"..build, "swap_lantern_on")
		owner.AnimState:Show("LANTERN_OVERLAY") 
    end
    -- lightup(inst)
end

local function onunequip(inst, owner)
    refreshstatus(inst)
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal")
    owner.AnimState:ClearOverrideSymbol("lantern_overlay")
	owner.AnimState:Hide("LANTERN_OVERLAY") 	
end

local function nofuel(inst)
    local owner = inst.components.inventoryitem and inst.components.inventoryitem.owner
    if owner then
        owner:PushEvent("torchranout", {torch = inst})
    end

    turnoff(inst)
end

local function takefuel(inst)
    if inst.components.equippable and inst.components.equippable:IsEquipped() then
        lightup(inst)
    end
end

local function fuelupdate(inst)
    local fuelpercent = inst.components.fueled:GetPercent()
    inst.Light:SetIntensity(Lerp(0.4, 0.6, fuelpercent))
    inst.Light:SetRadius(Lerp(3, 5, fuelpercent))
    inst.Light:SetFalloff(.9)
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()        
    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("lantern")
    inst.AnimState:SetBuild("lantern")
    inst.AnimState:PlayAnimation("idle_off")
	if SaveGameIndex:IsModeShipwrecked() or  SaveGameIndex:GetCurrentMode()== "porkland" then
		MakeInventoryFloatable(inst, "idle_off_water", "idle_off")
	end
    inst:AddTag("light")

    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")

    inst.components.inventoryitem:SetOnDroppedFn(ondropped)
    inst.components.inventoryitem:SetOnPickupFn(onpickup)
    --inst.components.inventoryitem:SetOnPickupFn(makesmalllight)
    --inst.components.inventoryitem:SetOnActiveItemFn(makesmalllight)
    inst.components.inventoryitem:SetOnPutInInventoryFn(onputininventory)    

    inst:AddComponent("equippable")

    inst:AddComponent("fueled")

    inst:AddComponent("machine")
    inst.components.machine.turnonfn = turnon
    inst.components.machine.turnofffn = turnoff
    inst.components.machine.cooldowntime = 0
    inst.components.machine.caninteractfn = function() return not inst.components.fueled:IsEmpty() and (inst.components.inventoryitem.owner == nil or inst.components.equippable.isequipped) end


    inst.components.fueled.fueltype = "CAVE"
    inst.components.fueled:InitializeFuelLevel(TUNING.LANTERN_LIGHTTIME)
    inst.components.fueled:SetDepletedFn(nofuel)
    inst.components.fueled:SetUpdateFn(fuelupdate)
    inst.components.fueled.ontakefuelfn = takefuel
    inst.components.fueled.accepting = true

    inst.entity:AddLight()
    inst.Light:SetColour(180/255, 195/255, 150/255)

    fuelupdate(inst)

    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip ) 

    inst:ListenForEvent( "startrowing", function(inst,data) 
        --print("start rowing!!")
        turnoff(inst)
        end, inst)  

    inst:ListenForEvent( "stoprowing", function(inst, data) 
        --print("stop rowing!!")
        turnon(inst)
        end, inst)

	inst:AddComponent("dsskins")
	inst.components.dsskins.skins = lantern_skins
	
	inst:ListenForEvent("unequipped", lantern_off)
	inst:ListenForEvent("onremove", lantern_off)
    inst.OnLoad = OnLoad

    return inst
end


return Prefab( "common/inventory/lantern", fn, assets) 
